@@ -618,7 +618,7 @@ void draw_form_entry(struct terminal *t, struct f_data_c *f, struct link *l)
 			while (ln->st && sl) sl--, ln++;
 			for (; ln->st && y < l->pos[0].y + yp - vy + form->rows; ln++, y++) {
 				for (i = 0; i < form->cols; i++) {
-					if (x+i >= xp && y >= yp && x < xp+xw && y < yp+yw) {
+					if (x+i >= xp && y >= yp && x+i < xp+xw && y < yp+yw) {
 						if (fs->value && i >= -fs->vpos && i + fs->vpos < ln->en - ln->st) set_only_char(t, x+i, y, ln->st[i + fs->vpos]);
 						else set_only_char(t, x+i, y, '_');
 					}
@@ -626,7 +626,7 @@ void draw_form_entry(struct terminal *t, struct f_data_c *f, struct link *l)
 			}
 			for (; y < l->pos[0].y + yp - vy + form->rows; y++) {
 				for (i = 0; i < form->cols; i++) {
-					if (x+i >= xp && y >= yp && x < xp+xw && y < yp+yw)
+					if (x+i >= xp && y >= yp && x+i < xp+xw && y < yp+yw)
 						set_only_char(t, x+i, y, '_');
 				}
 			}
@@ -1165,26 +1165,35 @@ void encode_string(unsigned char *name, unsigned char **data, int *len)
 	}
 }
 
-void encode_controls(struct list_head *l, unsigned char **data, int *len)
+void encode_controls(struct list_head *l, unsigned char **data, int *len,
+		     int cp_from, int cp_to)
 {
 	struct submitted_value *sv;
 	int lst = 0;
+	char *p2;
+	struct conv_table *convert_table = get_translation_table(cp_from, cp_to);
 	*len = 0;
 	*data = init_str();
 	foreach(sv, *l) {
 		unsigned char *p = sv->value;
+		struct document_options o;
+		o.plain = 1;
+		d_opt = &o;
 		if (sv->type == FC_TEXTAREA) p = encode_textarea(sv->value);
 		if (lst) add_to_str(data, len, "&"); else lst = 1;
 		encode_string(sv->name, data, len);
 		add_to_str(data, len, "=");
-		encode_string(p, data, len);
+		p2 = convert_string(convert_table, p, strlen(p));
+		encode_string(p2, data, len);
+		mem_free(p2);
 		if (sv->type == FC_TEXTAREA) mem_free(p);
 	}
 }
 
 #define BL	32
 
-void encode_multipart(struct session *ses, struct list_head *l, unsigned char **data, int *len, unsigned char *bound)
+void encode_multipart(struct session *ses, struct list_head *l, unsigned char **data, int *len,
+		      unsigned char *bound, int cp_from, int cp_to)
 {
 	int *nbp, *bound_ptrs = DUMMY;
 	int nbound_ptrs = 0;
@@ -1192,6 +1201,8 @@ void encode_multipart(struct session *ses, struct list_head *l, unsigned char **
 	struct submitted_value *sv;
 	int i, j;
 	int flg = 0;
+	char *p;
+	struct conv_table *convert_table = get_translation_table(cp_from, cp_to);
 	memset(bound, 'x', BL);
 	*len = 0;
 	*data = init_str();
@@ -1215,8 +1226,14 @@ void encode_multipart(struct session *ses, struct list_head *l, unsigned char **
 				   Netscape does the same */
 		}
 		add_to_str(data, len, "\"\r\n\r\n");
-		if (sv->type != FC_FILE) add_to_str(data, len, sv->value);
-		else {
+		if (sv->type != FC_FILE) {
+			struct document_options o;
+			o.plain = 1;
+			d_opt = &o;
+			p = convert_string(convert_table, sv->value, strlen(sv->value));
+			add_to_str(data, len, p);
+			mem_free(p);
+		} else {
 			int fh, rd;
 #define F_BUFLEN 1024
 			unsigned char buffer[F_BUFLEN];
@@ -1279,6 +1296,7 @@ unsigned char *get_form_url(struct session *ses, struct f_data_c *f, struct form
 	unsigned char bound[BL];
 	int len;
 	unsigned char *go = NULL;
+	int cp_from, cp_to;
 	if (!form) return NULL;
 	if (form->type == FC_RESET) {
 		reset_form(f, form->form_num);
@@ -1286,11 +1304,12 @@ unsigned char *get_form_url(struct session *ses, struct f_data_c *f, struct form
 	}
 	if (!form->action) return NULL;
 	get_succesful_controls(f, form, &submit);
+	cp_from = ses->term->spec->charset;
+	cp_to = f->f_data->cp;
 	if (form->method == FM_GET || form->method == FM_POST)
-		encode_controls(&submit, &data, &len);
-	else {
-		encode_multipart(ses, &submit, &data, &len, bound);
-	}
+		encode_controls(&submit, &data, &len, cp_from, cp_to);
+	else
+		encode_multipart(ses, &submit, &data, &len, bound, cp_from, cp_to);
 	if (!data) goto ff;
 	if (form->method == FM_GET) {
 		if ((go = mem_alloc(strlen(form->action) + 1 + len + 1))) {
@@ -1301,7 +1320,7 @@ unsigned char *get_form_url(struct session *ses, struct f_data_c *f, struct form
 				pos = stracpy(pos);
 				*poss = 0;
 			}
-			if (!*go || go[strlen(go) - 1] != '?') strcat(go, "?");
+			strcat(go, "?");
 			strcat(go, data);
 			if (pos) strcat(go, pos), mem_free(pos);
 		}
@@ -1983,12 +2002,22 @@ void send_event(struct session *ses, struct event *ev)
 				mem_free(s);
 				goto x;
 			}
+			case ACT_GOTO_URL_CURRENT_LINK: {
+				unsigned char url[MAX_STR_LEN];
+				if (!get_current_link_url(ses, url, sizeof url)) goto quak;
+				dialog_goto_url(ses, url);
+				goto x;	
+			}
 			case ACT_ADD_BOOKMARK:
 				if (!anonymous) launch_bm_add_doc_dialog(ses->term, NULL, ses);
 				goto x;
 			case ACT_BOOKMARK_MANAGER:
 				if (!anonymous) menu_bookmark_manager(ses->term, NULL, ses);
 				goto x;
+			case ACT_REALLYQUIT:
+				reallyquit:
+				exit_prog(ses->term, (void *)1, ses);
+				goto x;
 			case ACT_QUIT:
 		  		quit:
 				exit_prog(ses->term, (void *)(ev->x == KBD_CTRL_C), ses);
@@ -2004,9 +2033,20 @@ void send_event(struct session *ses, struct event *ev)
 				html_interpret(ses);
 				draw_formatted(ses);
 				goto x;
+			case ACT_TOGGLE_DISPLAY_TABLES:
+				ses->ds.tables = !ses->ds.tables;
+				html_interpret(ses);
+				draw_formatted(ses);
+				goto x;
 			case ACT_TOGGLE_HTML_PLAIN:
 				toggle(ses, ses->screen, 0);
 				goto x;
+			case ACT_OPEN_NEW_WINDOW:
+				open_in_new_window(ses->term, send_open_new_xterm, ses);
+				goto x;
+			case ACT_OPEN_LINK_IN_NEW_WINDOW:
+				open_in_new_window(ses->term, send_open_in_new_xterm, ses);
+				goto x;
 			default:
 				if (ev->x == KBD_CTRL_C) goto quit;
 				if (ev->y & KBD_ALT) {
@@ -2428,7 +2468,8 @@ void head_msg(struct session *ses)
 		return;
 	}
 	if (!find_in_cache(cur_loc(ses)->vs.url, &ce)) {
-		ss = s = stracpy(ce->head);
+		if (ce->head) ss = s = stracpy(ce->head);
+		else s = ss = stracpy("");
 		len = strlen(s) - 1;
 		if (len > 0) {
 			while (ss = strstr(s, "\r\n")) memmove(ss, ss + 1, strlen(ss));
